#ifndef UTILITIES_H_INCLUDED
#define UTILITIES_H_INCLUDED

#include <vector>
#include <algorithm>
#include <iterator>
#include <tuple>

#include "divisors_table.h"

typedef std::tuple<unsigned int, unsigned int, unsigned int, unsigned int> Tiling;
typedef std::vector< Tiling > Tiling_2d;

//! Read image and check number of channels
int load_image(const char *name
,   std::vector<float> &img
,   unsigned * width
,   unsigned * height
,   unsigned * chnls
);

//! Read image and check number of channels
int load_image(const char *name
,   std::vector<float> &img
,   unsigned  width
,   unsigned  height
,   unsigned  chnls
);

//! Write image
int save_image(
    char* name
,   std::vector<float> &img
,   const unsigned width
,   const unsigned height
,   const unsigned chnls
);

//! Read image and check number of channels
int load_image_IIO(const char *name
,   std::vector<float> &img
,   unsigned * width
,   unsigned * height
,   unsigned * chnls
);

//! Read image and check number of channels
int load_image_IIO(const char *name
,   std::vector<float> &img
,   unsigned width
,   unsigned height
,   unsigned chnls
);

//! Write image
int save_image_IIO(
    char* name
,   std::vector<float> &img
,   const unsigned width
,   const unsigned height
,   const unsigned chnls
);

//! Add noise
void add_noise(
    const std::vector<float> &img
,   std::vector<float> &img_noisy
,   const float sigma
);

//! Best load_balance decomposition from image size and
//! number of processor
int load_balancing_2d(unsigned int nb_threads,
        unsigned int width,
        unsigned int height,
        Tiling_2d &decomposition);

//! Add boundaries by symmetry (NxN)
void symmetrize(
    const std::vector<float> &img
,   std::vector<float> &img_sym
,   const unsigned width
,   const unsigned height
,   const unsigned chnls
,   const unsigned N
);

//! Add boundaries by symetry (PxQ)
void symmetrize(
    const std::vector<float> &img
,   std::vector<float> &img_sym
,   const unsigned width
,   const unsigned height
,   const unsigned chnls
,   const unsigned P
,   const unsigned Q
);

//! Add boundaries by symetry (PxQ) onto a region of support 
void symmetrize(
    const std::vector<float> &img
,   std::vector<float> &img_sym
,   const unsigned width
,   const unsigned height
,   const unsigned chnls
,   const unsigned P
,   const unsigned Q
,   const Tiling &region
);

//! Compute the PSNR and RMSE between img_1 and img_2
int compute_psnr(
    const std::vector<float> &img_1
,   const std::vector<float> &img_2
,   float *psnr
,   float *rmse
);

//! Transform the color space of the image
int color_space_transform(
    std::vector<float> &img
,   const unsigned color_space
,   const unsigned width
,   const unsigned height
,   const unsigned chnls
,   const bool rgb2yuv
);

//! Estimate sigma on each channel according to the choice of the color_space
int estimate_sigma(
    const float sigma
,   std::vector<float> &sigma_table
,   const unsigned chnls
,   const unsigned color_space
);

#endif // UTILITIES_H_INCLUDED
